/*	Formatright  2009, ConnorMcLeod

	Half Life Media Player is free software;
	you can redistribute it and/or modify it under the terms of the
	GNU General Public License as published by the Free Software Foundation.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Half Life Media Player; if not, write to the
	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.
*/

#include <amxmodx>

#define VERSION "3.0.0"

#define GROUP_MAX_LENGTH	64

#define RADIO_MAX_LENGTH	64
#define URL_MAX_LENGTH	192

#define MAX_PLAYERS	32

#define DEFAULT_VOLUME 40

#define SetIdBits(%1,%2)		%1 |= 1<<(%2 % 32)
#define ClearIdBits(%1,%2)	%1 &= ~( 1<<(%2 % 32) )
#define GetIdBits(%1,%2)		%1 & 1<<(%2 % 32)

enum _:Radios
{
	RadioName[RADIO_MAX_LENGTH],
	RadioUrl[URL_MAX_LENGTH]
}

enum _:Group
{
	GroupName[GROUP_MAX_LENGTH],
	Array:GroupArrayOffset
}

enum _:MenuSettings
{
	mGroups,
	mRadios,
	mConfig
}

new Array:g_aGroups

new g_iGroupsCount

new g_iMenuOption[MAX_PLAYERS+1]
new g_iMenuPosition[MAX_PLAYERS+1]
new g_bRepeat
new g_iVolume[MAX_PLAYERS+1] = {DEFAULT_VOLUME, ...}
new Array:g_aPlayerGroup[MAX_PLAYERS+1]

public plugin_init()
{
	register_plugin("Half Life Media Player", VERSION, "ConnorMcLeod")
	register_dictionary("common.txt")
	register_dictionary("hlmp.txt")

	register_menucmd(register_menuid("HLMP"), 0x3FF, "HlmpMenuAction")

	register_clcmd("hlmp_menu", "ClientCommand_HlmpMenu")
	register_clcmd("say /hlmp", "ClientCommand_HlmpMenu")
	register_clcmd("say /music", "ClientCommand_HlmpMenu")
	register_clcmd("say /radio", "ClientCommand_HlmpMenu")
	register_clcmd("say_team /hlmp", "ClientCommand_HlmpMenu")
	register_clcmd("say_team /music", "ClientCommand_HlmpMenu")
	register_clcmd("say_team /radio", "ClientCommand_HlmpMenu")

	register_clcmd("say /stop", "ClientCommand_StopMusic")
	register_clcmd("say_team /stop", "ClientCommand_StopMusic")

	Read_ConfigFile()
}

public client_putinserver(id)
{
	g_iMenuOption[id] = mGroups

	new szVolume[4], szRepeat[2]
	get_user_info(id, "hlmp_volume", szVolume, charsmax(szVolume))
	get_user_info(id, "hlmp_repeat", szRepeat, charsmax(szRepeat))
	if( szVolume[0] )
	{
		g_iVolume[id] = clamp(str_to_num(szVolume), 0, 100)
	}
	else
	{
		g_iVolume[id] = DEFAULT_VOLUME
	}

	if( szRepeat[0] == '1' )
	{
		SetIdBits(g_bRepeat, id)
	}
	else
	{
		ClearIdBits(g_bRepeat, id)
	}
}

Read_ConfigFile()
{
	new szConfigFile[64]
	get_localinfo("amxx_configsdir", szConfigFile, charsmax(szConfigFile))
	format(szConfigFile, charsmax(szConfigFile), "%s/mediafiles.ini", szConfigFile)

	new iFile = fopen(szConfigFile, "rt")

	if(!iFile)
	{
		return
	}

	g_aGroups = ArrayCreate(Group)

	new iGroup
	new szText[Radios+16], RadioDatas[Radios], GroupDatas[Group]
	new Array:aRadiosDatas

	while(!feof(iFile))
	{
		fgets(iFile, szText, charsmax(szText))
		trim( szText )

		if(!szText[0] || szText[0] == '#' || szText[0] == ';' || (szText[0] == '/' && szText[1] == '/'))
		{
			continue
		}

		if( szText[0] == '[' )
		{
			iGroup++
			aRadiosDatas = ArrayCreate( Radios )

			strtok(szText[1], GroupDatas[GroupName], charsmax(GroupDatas[GroupName]), RadioDatas, 1, ']', 0)
			GroupDatas[GroupArrayOffset] = _:aRadiosDatas

			ArrayPushArray(g_aGroups, GroupDatas)
		}
		else if( iGroup )
		{
			parse(	szText,
					RadioDatas[RadioName], charsmax(RadioDatas[RadioName]), 
					RadioDatas[RadioUrl], charsmax(RadioDatas[RadioUrl])	)

			ArrayPushString(aRadiosDatas, RadioDatas)
		}
	}
	fclose(iFile)

	g_iGroupsCount = ArraySize(g_aGroups)
}

public ClientCommand_HlmpMenu(id)
{
	DisplayMenu(id, g_iMenuPosition[id] = 0)
}


DisplayMenu(id, iPos = 0)
{
	new szMenu[1024], n
	new iKeys = MENU_KEY_6|MENU_KEY_7|MENU_KEY_0

	n = formatex(szMenu[n], charsmax(szMenu)-n, "\rHL Media Player (HLMP %s)^n", VERSION)

	switch( g_iMenuOption[id] )
	{
		case mGroups:
		{
			new iStart = iPos * 5
			new iStop = min(iStart + 5 , g_iGroupsCount)
			new aGroup[Group]

			n += formatex(szMenu[n], charsmax(szMenu)-n, "\d%L^n^n", id, "HLMP_GROUPS")
			
			for(new i=iStart, j; i<iStop; i++)
			{
				iKeys |= (1<<j)
				ArrayGetArray(g_aGroups, i, aGroup)
				n += formatex(szMenu[n], charsmax(szMenu)-n, "\r%d. \y%s^n", ++j, aGroup[GroupName])
			}
			iPos = iStop - iStart
			if( iPos < 5 )
			{
				for(new i; i< 5 - iPos; i++)
				{
					n += formatex(szMenu[n], charsmax(szMenu)-n, "^n")
				}
			}

			n += formatex(szMenu[n], charsmax(szMenu)-n, "^n\w6. \y%L^n", id, "HLMP_CONFIG")
			n += formatex(szMenu[n], charsmax(szMenu)-n, "\w7. \y%L^n", id, "HLMP_STOPMUSIC")

			if( iStart )
			{
				iKeys |= MENU_KEY_8
				n += formatex(szMenu[n], charsmax(szMenu)-n, "^n\w8. %L", id, "BACK")
			}

			if( iStop < g_iGroupsCount )
			{
				iKeys |= MENU_KEY_9
				n += formatex(szMenu[n], charsmax(szMenu)-n, "^n\w9. %L", id, "MORE")
			}
		}
		case mRadios:
		{
			new Array:aRadios = g_aPlayerGroup[id]
			new aCurRadio[Radios]
			new iRadiosNum = ArraySize(aRadios)

			new iStart = iPos * 5
			new iStop = min(iStart + 5 , iRadiosNum)

			n += formatex(szMenu[n], charsmax(szMenu)-n, "\d%L^n^n", id, "HLMP_RADIOS")

			for(new i=iStart, j; i<iStop; i++)
			{
				iKeys |= (1<<j)
				ArrayGetArray(aRadios, i, aCurRadio)
				n += formatex(szMenu[n], charsmax(szMenu)-n, "\r%d. \y%s^n", ++j, aCurRadio[RadioName])
			}
			iPos = iStop - iStart
			if( iPos < 5 )
			{
				for(new i; i< 5 - iPos; i++)
				{
					n += formatex(szMenu[n], charsmax(szMenu)-n, "^n")
				}
			}

			n += formatex(szMenu[n], charsmax(szMenu)-n, "^n\w6. \y%L^n", id, "HLMP_CONFIG")
			n += formatex(szMenu[n], charsmax(szMenu)-n, "\w7. \y%L^n", id, "HLMP_STOPMUSIC")

			if( iStart )
			{
				iKeys |= MENU_KEY_8
				n += formatex(szMenu[n], charsmax(szMenu)-n, "^n\w8. %L", id, "BACK")
			}

			if( iStop < iRadiosNum )
			{
				iKeys |= MENU_KEY_9
				n += formatex(szMenu[n], charsmax(szMenu)-n, "^n\w9. %L", id, "MORE")
			}

		}
		case mConfig:
		{
			iKeys |= MENU_KEY_1|MENU_KEY_2|MENU_KEY_3|MENU_KEY_4|MENU_KEY_6|MENU_KEY_7

			n += formatex(szMenu[n], charsmax(szMenu)-n, "\d%L^n^n", id, "HLMP_CONFIG")

			n += formatex(szMenu[n], charsmax(szMenu)-n, "\w1. \y%L\R%L^n", id, "HLMP_REPEAT", id, GetIdBits(g_bRepeat, id) ? "ON" : "OFF")
			n += formatex(szMenu[n], charsmax(szMenu)-n, "\w2. \y%L +\R%d^n", id, "HLMP_VOLUME", g_iVolume[id])
			n += formatex(szMenu[n], charsmax(szMenu)-n, "\w3. \y%L -^n", id, "HLMP_VOLUME")
			n += formatex(szMenu[n], charsmax(szMenu)-n, "\w4. \y%L^n", id, "HLMP_ABOUT")

			n += formatex(szMenu[n], charsmax(szMenu)-n, "^n\w6. \yGroups^n")
			n += formatex(szMenu[n], charsmax(szMenu)-n, "\w7. \y%L^n", id, "HLMP_STOPMUSIC")
		}
	}

	n += formatex(szMenu[n], charsmax(szMenu)-n, "^n\w0. %L", id, "EXIT")

	show_menu(id, iKeys, szMenu, 30, "HLMP")

	return PLUGIN_HANDLED
}

public HlmpMenuAction(id, iKey)
{
	switch( g_iMenuOption[id] )
	{
		case mGroups:
		{
			switch( iKey )
			{
				case 0..4:
				{
					new aGroup[Group]

					ArrayGetArray(g_aGroups, g_iMenuPosition[id]*5 + iKey, aGroup)
					g_aPlayerGroup[id] = aGroup[GroupArrayOffset]

					g_iMenuOption[id] = mRadios
					DisplayMenu(id, g_iMenuPosition[id] = 0)
				}
				case 5:
				{
					g_iMenuOption[id] = mConfig
					DisplayMenu(id, g_iMenuPosition[id] = 0)
				}
				case 6:
				{
					ClientCommand_StopMusic(id)
				}
				case 7:
				{
					if( --g_iMenuPosition[id] < 0 )
					{
						g_iMenuPosition[id] = 0
					}
					DisplayMenu(id, g_iMenuPosition[id])
				}
				case 8:
				{
					if( ++g_iMenuPosition[id] >= g_iGroupsCount / 5 )
					{
						g_iMenuPosition[id] = g_iGroupsCount / 5
					}
					DisplayMenu(id, g_iMenuPosition[id])
				}
				case 9:
				{
					return PLUGIN_HANDLED
				}
			}
		}
		case mRadios:
		{
			switch( iKey )
			{
				case 0..4:
				{
					PlayMusic(id, g_aPlayerGroup[id], g_iMenuPosition[id]*5 + iKey)
				}
				case 5:
				{
					g_iMenuOption[id] = mConfig
					DisplayMenu(id, g_iMenuPosition[id] = 0)
				}
				case 6:
				{
					ClientCommand_StopMusic(id)
				}
				case 7:
				{
					if( --g_iMenuPosition[id] < 0 )
					{
						g_iMenuPosition[id] = 0
					}
					DisplayMenu(id, g_iMenuPosition[id])
				}
				case 8:
				{
					new iSize = ArraySize(g_aPlayerGroup[id])
					if( ++g_iMenuPosition[id] >= iSize / 5 )
					{
						g_iMenuPosition[id] = iSize / 5
					}
					DisplayMenu(id, g_iMenuPosition[id])
				}
				case 9:
				{
					return PLUGIN_HANDLED
				}
			}
		}
		case mConfig:
		{
			switch( iKey )
			{
				case 0:
				{
					if( GetIdBits(g_bRepeat, id) )
					{
						ClearIdBits(g_bRepeat, id)
						client_cmd(id, "setinfo ^"hlmp_repeat^" ^"0^"")
					}
					else
					{
						SetIdBits(g_bRepeat, id)
						client_cmd(id, "setinfo ^"hlmp_repeat^" ^"1^"")
					}
					client_print(id, print_chat, "%L", id, "HLMP_CONFIGMENUTIP")
					DisplayMenu(id)
				}
				case 1:
				{
					if( (g_iVolume[id] += 5) > 100)
					{
						g_iVolume[id] = 100
					}
					client_print(id, print_chat, "%L", id, "HLMP_CONFIGMENUTIP")
					client_cmd(id, "setinfo ^"hlmp_volume^" ^"%d^"", g_iVolume[id])
					DisplayMenu(id)
				}
				case 2:
				{
					if( (g_iVolume[id] -= 5) < 0)
					{
						g_iVolume[id] = 0
					}
					client_cmd(id, "setinfo ^"hlmp_volume^" ^"%d^"", g_iVolume[id])
					client_print(id, print_chat, "%L", id, "HLMP_CONFIGMENUTIP")
					DisplayMenu(id)
				}
				case 3:
				{
					client_print(id, print_chat, "HL Media Player %s by ConnorMcLeod, download link in console.", VERSION)
					client_print(id, print_center, "HL Media Player %s by ConnorMcLeod", VERSION)
					client_print(id, print_console, "http://forums.alliedmods.net/showthread.php?p=833070#post833070")
				}
				case 5:
				{
					g_iMenuOption[id] = mGroups
					DisplayMenu(id, g_iMenuPosition[id] = 0)
				}
				case 6:
				{
					ClientCommand_StopMusic(id)
				}
				case 9:
				{
					return PLUGIN_HANDLED
				}
				default:
				{
					DisplayMenu(id)
				}
			}
		}
	}
	return PLUGIN_HANDLED
}

PlayMusic(id, Array:aGroup, iRadio)
{
	new Radio[Radios]
	ArrayGetArray(aGroup, iRadio, Radio)

	new szMotd[1024], n

	n = formatex(szMotd[n], charsmax(szMotd)-n, "<html><head><meta http-equiv=^"content-type^" content=^"text/html; charset=UTF-8^"></head><body bgcolor=^"#000000^" align=^"center^"><span style=^"color: #FFB000; font-size: 9pt^">Now playing: %s <br>", Radio[RadioName])
	n += formatex(szMotd[n], charsmax(szMotd)-n, "<object classid=CLSID:6BF52A52-394A-11d3-B153-00C04F79FAA6 codebase=http://www.microsoft.com/ntserver/netshow/download/en/nsmp2inf.cab#Version=5,1,51,415 type=application/x-oleobject name=msplayer width=256 height=65 align=^"middle^" id=msplayer>")
	n += formatex(szMotd[n], charsmax(szMotd)-n, "<param name=^"enableContextMenu^" value=^"0^"><param name=^"stretchToFit^" value=^"1^">")
	if(GetIdBits(g_bRepeat, id))
	{
		n += formatex(szMotd[n], charsmax(szMotd)-n, "<param name=^"AutoRewind^" value=^"1^">")
	}
	n += formatex(szMotd[n], charsmax(szMotd)-n, "<param name=^"Volume^" value=^"%d^">", g_iVolume[id])
	n += formatex(szMotd[n], charsmax(szMotd)-n, "<param name=^"AutoStart^" value=^"1^"><param name=^"URL^" value=^"%s^">", Radio[RadioUrl])
	n += formatex(szMotd[n], charsmax(szMotd)-n, "<param name=^"uiMode^" value=^"full^"><param name=^"width^" value=^"256^"><param name=^"height^" value=^"65^">")
	n += formatex(szMotd[n], charsmax(szMotd)-n, "<param name=^"TransparentAtStart^" value=^"1^"></object><br>^"%L^"</span>", id, "HLMP_CLOSEWINDOW")
	n += formatex(szMotd[n], charsmax(szMotd)-n, "</body></html>")
	show_motd(id, szMotd, "HL Media Player")
}

public ClientCommand_StopMusic(id)
{
	new szMotd[256]
	formatex(szMotd, charsmax(szMotd), "<html><head><meta http-equiv=^"content-type^" content=^"text/html; charset=UTF-8^"></head><body bgcolor=^"#000000^" align=^"center^"><span style=^"color: #FFB000; font-size: 9pt^">^"%L^"</span></body></html>", id, "HLMP_CLOSEWINDOW")
	show_motd(id, szMotd, "HL Media Player")
	return PLUGIN_HANDLED
}